"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toArray(arr) { return _arrayWithHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/** Visitor factory for babel, converting React.createElement(...) to <jsx ...>...</jsx>
 *
 * What we want to handle here is this CallExpression:
 *
 *     React.createElement(
 *       type: StringLiteral|Identifier|MemberExpression,
 *       [props: ObjectExpression|Expression],
 *       [...children: StringLiteral|Expression]
 *     )
 *
 * Any of those arguments might also be missing (undefined) and/or invalid. */
function _default(_ref) {
  var t = _ref.types;

  /** Get a JSXElement from a CallExpression
   * Returns null if this impossible */
  function getJSXNode(node) {
    // only transform "React.createElement" Expressions
    if (!isReactCreateElement(node)) return null; //nameNode and propsNode may be undefined, getJSX* need to handle that

    var _node$arguments = _toArray(node.arguments),
        nameNode = _node$arguments[0],
        propsNode = _node$arguments[1],
        childNodes = _node$arguments.slice(2);

    var name = getJSXName(nameNode);
    if (name === null) return null; //name is required

    var children = isEmptyChildren(childNodes) ? [] : getJSXChildren(childNodes);
    if (children === null) return null; //no children → [], invalid → null

    if (isReactFragment(nameNode)) {
      return jSXFragment(children);
    }

    var props = getJSXProps(propsNode);
    if (props === null) return null; //no props → [], invalid → null
    // console.log("props", name, props, children)
    // self-closing tag if no children

    var selfClosing = children.length === 0;
    var startTag = t.jSXOpeningElement(name, props, selfClosing);
    var endTag = selfClosing ? null : t.jSXClosingElement(name);
    return t.jSXElement(startTag, endTag, children, selfClosing);
  }
  /** Get a JSXIdentifier or JSXMemberExpression from a Node of known type.
   * Returns null if a unknown node type, null or undefined is passed. */


  function getJSXName(node) {
    if (node == null) return null;
    var name = getJSXIdentifier(node);
    if (name !== null) return name;
    if (!t.isMemberExpression(node)) return null;
    var object = getJSXName(node.object);
    var property = getJSXName(node.property);
    if (object === null || property === null) return null;
    return t.jSXMemberExpression(object, property);
  }
  /** Get a array of JSX(Spread)Attribute from a props ObjectExpression.
   * Handles the _extends Expression babel creates from SpreadElement nodes.
   * Returns null if a validation error occurs. */


  function getJSXProps(node) {
    if (node == null || isNullLikeNode(node)) return [];

    if (t.isCallExpression(node) && t.isIdentifier(node.callee, {
      name: "_extends"
    })) {
      var props = node.arguments.map(getJSXProps); //if calling this recursively works, flatten.

      if (props.every(function (prop) {
        return prop !== null;
      })) return [].concat.apply([], props);
    }

    if (!t.isObjectExpression(node) && t.isExpression(node)) return [t.jSXSpreadAttribute(node)];
    if (!isPlainObjectExpression(node)) return null;
    return node.properties.map(function (prop) {
      return t.isObjectProperty(prop) ? t.jSXAttribute(getJSXIdentifier(prop.key), getJSXAttributeValue(prop.value)) : t.jSXSpreadAttribute(prop.argument);
    });
  }

  function getJSXChild(node) {
    if (t.isStringLiteral(node)) {
      // Solidjs reads from node.extra.raw, but jSXText doesn't create it. So we do it by hand.
      return _objectSpread(_objectSpread({}, t.jSXText(node.value)), {}, {
        extra: {
          raw: node.value
        }
      });
    }

    ;
    if (isReactCreateElement(node)) return getJSXNode(node);
    if (t.isExpression(node)) return t.jSXExpressionContainer(node);
    return null;
  }

  function getJSXChildren(nodes) {
    var children = nodes.filter(function (node) {
      return !isNullLikeNode(node);
    }).map(getJSXChild);
    if (children.some(function (child) {
      return child == null;
    })) return null;
    return children;
  }

  function getJSXIdentifier(node) {
    //TODO: JSXNamespacedName
    if (t.isIdentifier(node)) return t.jSXIdentifier(node.name);
    if (t.isStringLiteral(node)) return t.jSXIdentifier(node.value);
    return null;
  }

  function getJSXAttributeValue(node) {
    if (t.isStringLiteral(node)) return node;
    if (t.isJSXElement(node)) return node;
    if (t.isExpression(node)) return t.jSXExpressionContainer(node);
    return null;
  }
  /** tests if a node is a CallExpression with callee “React.createElement” */


  var isReactCreateElement = function isReactCreateElement(node) {
    return t.isCallExpression(node) && t.isMemberExpression(node.callee) && (t.isIdentifier(node.callee.object, {
      name: "React"
    }) || t.isIdentifier(node.callee.object, {
      name: "ReactDOMRe"
    })) && (t.isIdentifier(node.callee.property, {
      name: "createElement"
    }) || t.isIdentifier(node.callee.property, {
      name: "createElementVariadic"
    }) || t.isIdentifier(node.callee.property, {
      name: "createDOMElementVariadic"
    })) && !node.callee.computed;
  };
  /** tests if a node is a react fragment (<></>) */


  var isReactFragment = function isReactFragment(node) {
    return t.isMemberExpression(node) && t.isIdentifier(node.object, {
      name: "ReasonReact"
    });
  };
  /** Creates a JSX Fragment with given children */


  var jSXFragment = function jSXFragment(children) {
    return {
      type: "JSXFragment",
      openingElement: {
        type: "JSXOpeningFragment"
      },
      closingElement: {
        type: "JSXClosingFragment"
      },
      children: children
    };
  };
  /** tests if an array of nodes only contains empty ArrayExpressions */


  var isEmptyChildren = function isEmptyChildren(children) {
    return children.every(function (child) {
      return child.elements && child.elements.length === 0;
    });
  };
  /** Tests if a node is “null” or “undefined” */


  var isNullLikeNode = function isNullLikeNode(node) {
    return t.isNullLiteral(node) || t.isIdentifier(node, {
      name: "undefined"
    });
  };
  /** Tests if a node is an object expression with noncomputed, nonmethod attrs */


  var isPlainObjectExpression = function isPlainObjectExpression(node) {
    return t.isObjectExpression(node) && node.properties.every(function (m) {
      return t.isSpreadElement(m) || t.isObjectProperty(m, {
        computed: false
      }) && getJSXIdentifier(m.key) !== null && getJSXAttributeValue(m.value) !== null;
    });
  };

  return {
    visitor: {
      CallExpression: function CallExpression(path) {
        var node = getJSXNode(path.node);
        if (node === null) return null;

        if (node.length && node.length > 0) {
          path.replaceWithMultiple(node);
        } else {
          path.replaceWith(node);
        }
      }
    }
  };
}